@@ -59,7 +59,7 @@ autoplot.tbl_ts <- function(object, .vars = NULL, ...){
   
   p <- ggplot(object, eval_tidy(expr(aes(!!!aes_spec)))) + 
     geom_line(...) +
-    xlab(paste0(expr_text(index(object)), " [", format(interval(object)), "]"))
+    xlab(paste0(index_var(object), " [", format(interval(object)), "]"))
   
   if(nk > 1){
     p <- p + 
